@@ -1,8 +1,8 @@
 # $File: //member/autrijus/ExtUtils-AutoInstall/AutoInstall.pm $ 
-# $Revision: #9 $ $Change: 1323 $ $DateTime: 2002/10/11 09:40:48 $
+# $Revision: #15 $ $Change: 2801 $ $DateTime: 2002/12/20 03:42:23 $
 
 package ExtUtils::AutoInstall;
-$ExtUtils::AutoInstall::VERSION = '0.40';
+$ExtUtils::AutoInstall::VERSION = '0.43';
 
 use strict;
 
@@ -15,8 +15,8 @@ ExtUtils::AutoInstall - Automatic install of dependencies via CPAN
 
 =head1 VERSION
 
-This document describes version 0.40 of B<ExtUtils::AutoInstall>,
-released October 11, 2002.
+This document describes version 0.43 of B<ExtUtils::AutoInstall>,
+released December 20, 2002.
 
 =head1 SYNOPSIS
 
@@ -42,7 +42,7 @@ In F<Makefile.PL>:
 	-version	=> '0.40',	# required AutoInstall version
 	                                # usually 0.30 is sufficient
 	-config		=> {
-	    make_args	=> '--hello'	# option(s) for CPAN::Config 
+	    make_args	=> '--hello'	# option(s) for CPAN::Config
 	    force	=> 1,		# pseudo-option to force install
 	},
 	-core		=> [		# core modules; may also be 'all'
@@ -103,14 +103,14 @@ yes/no on each one's dependencies; the module writer may also supply a
 boolean value via C<-default> to specify the default choice.
 
 The B<Core Features> marked by the name C<-core> will double-check with
-the user, if the user chooses not to install the modules that belongs to
-it.  This differs with the pre-0.26 'silent install' behaviour.
+the user, if the user chooses not to install the modules that belong to
+it.  This differs from the pre-0.26 'silent install' behaviour.
 
 Starting from version 0.27, if C<-core> is set to the string C<all>
-(case-insensitive), every features will be considered mandatory.
+(case-insensitive), every feature will be considered mandatory.
 
 The dependencies are expressed as pairs of C<Module> => C<version>
-inside an a array reference.  If the order does not matter, and there
+inside an array reference.  If the order does not matter, and there
 are no C<-default>, C<-tests> or C<-skiptests> directives for that
 feature, you may also use a hash reference.
 
@@ -137,7 +137,7 @@ If you have the B<CPANPLUS> package installed in your system, it is
 preferred by default over B<CPAN>; it also accepts some extra options
 (e.g. C<-target =E<gt> 'skiptest', -skiptest =E<gt> 1> to skip testing).
 
-All modules scheduled to install will be deleted from C<%INC> first, so
+All modules scheduled to be installed will be deleted from C<%INC> first, so
 B<ExtUtils::MakeMaker> will check the newly installed modules.
 
 Additionally, you could use the C<make installdeps> target to install
@@ -157,12 +157,12 @@ F<Makefile.PL>), you have to declare those two handlers I<before> the
 C<use> statement for them to take effect.
 
 If the user did not choose to install a module or it already exists on
-the system, neither handlers are invoked.  Both handlers are invoked
-exactly once for each module's attempted installation.
+the system, neither of the handlers is invoked.  Both handlers are invoked
+exactly once for each module when installation is attempted.
 
 C<MY::preinstall> takes two arguments, C<$module_name> and C<$version>;
 if it returns a false value, installation for that module will be
-    skipped, and C<MY::postinstall> won't be called at all.
+skipped, and C<MY::postinstall> won't be called at all.
 
 C<MY::postinstall> takes three arguments, C<$module_name>, C<$version>
 and C<$success>.  The last one denotes whether the installation
@@ -171,6 +171,25 @@ means failure during install, and C<undef> means that the installation
 was not attempted at all, possibly due to connection problems, or that
 module does not exist on CPAN at all.
 
+=head2 Customized C<MY::postamble>
+
+Starting from version 0.43, B<ExtUtils::AutoInstall> supports modules
+that require a C<MY::postamble> subroutine in their F<Makefile.PL>.
+The user-defined C<MY::postamble>, if present, is responsible to call
+C<ExtUtils::AutoInstall::postamble> and include the output in its
+return value.
+
+For example, the B<DBD::*> (database driver) modules for the Perl DBI
+are required to include the postamble generated by the function
+C<dbd_postamble>, so their F<Makefile.PL> may contain lines like this:
+
+    sub MY::postamble {
+	return &ExtUtils::AutoInstall::postamble . &dbd_postamble;
+    }
+
+Note that the B<ExtUtils::AutoInstall> module does not export the
+C<postamble> function, so the name should always be fully qualified.
+
 =head1 CAVEATS
 
 B<ExtUtils::AutoInstall> will add C<UNINST=1> to your B<make install>
@@ -181,13 +200,10 @@ cause dependency problems if you are using a fine-tuned directory
 structure for your site.  Please consult L<CPAN/FAQ> for an explanation
 in detail.
 
-If B<Sort::Versions> is available, it will be used to compare the
-required version with the existing module's version and the CPAN
-module's.  Otherwise it silently falls back to use I<cmp>.  This may
-cause inconsistent behaviours in pathetic situations.
-
-B<Inline::MakeMaker> is not happy with this module, since it prohibits
-competing C<MY::postamble> functions.  Patches welcome.
+If either B<version> or B<Sort::Versions> is available, they will be
+used to compare the required version with the existing module's version
+and the CPAN module's.  Otherwise it silently falls back to use I<cmp>.
+This may cause inconsistent behaviours in pathetic situations.
 
 =head1 NOTES
 
@@ -199,7 +215,7 @@ B<AppConfig>) is not very desirable either.
 
 The current compromise is to add the bootstrap code listed in the
 L</SYNOPSIS> before every script, but that does not look pretty, and
-will not work without an internet connection.
+will not work without an Internet connection.
 
 Since we do not want all future options of B<ExtUtils::AutoInstall> to
 be painfully detected manually like above, this module provides a
@@ -226,11 +242,23 @@ my %FeatureMap = (
 
 # various lexical flags
 my (@Missing, @Existing, %DisabledTests, $UnderCPAN, $HasCPANPLUS);
-my ($Config, $CheckOnly, $SkipInstall, $AcceptDefault, $TestOnly); 
+my ($Config, $CheckOnly, $SkipInstall, $AcceptDefault, $TestOnly);
+my ($PostambleActions, $PostambleUsed);
 
 $AcceptDefault = 1 unless -t STDIN; # non-interactive session
 _init();
 
+sub missing_modules {
+    return @Missing;
+}
+
+sub do_install {
+    __PACKAGE__->install(
+	[ UNIVERSAL::isa($Config, 'HASH') ? %{$Config} : @{$Config}],
+	@Missing,
+    );
+}
+
 # initialize various flags, and/or perform install
 sub _init {
     foreach my $arg (@ARGV, split(/[\s\t]+/, $ENV{PERL_EXTUTILS_AUTOINSTALL} || '')) {
@@ -372,7 +400,7 @@ sub import {
     chdir $cwd;
 
     # import to main::
-    no strict 'refs'; 
+    no strict 'refs';
     *{'main::WriteMakefile'} = \&Write;
 }
 
@@ -578,7 +606,7 @@ sub _under_cpan {
 
     my $cwd  = File::Spec->canonpath(Cwd::cwd());
     my $cpan = File::Spec->canonpath($CPAN::Config->{cpan_home});
-    
+
     return (index($cwd, $cpan) > -1);
 }
 
@@ -613,7 +641,7 @@ sub _connected_to {
     my $site = shift;
 
     return (
-	( _load('Socket') and Socket::inet_aton('$site') ) or _prompt(qq(
+	( _load('Socket') and Socket::inet_aton($site) ) or _prompt(qq(
 *** Your host cannot resolve the domain name '$site', which
     probably means the Internet connections are unavailable.
 ==> Should we try to install the required module(s) anyway?), 'n'
@@ -654,10 +682,19 @@ sub _version_check {
 
     $cur =~ s/\s+$//;
 
-    if ($Sort::Versions::VERSION or defined(_load('Sort::Versions'))) {
-	# use Sort::Versions as the sorting algorithm for a.b.c versions
-	return ((Sort::Versions::versioncmp($cur, $min) != -1) ? $cur : undef)
-	    if $cur =~ /\..*\./ or $min =~ /\..*\./;
+    # check for version numbers that are not in decimal format
+    if (ref($cur) or ref($min) or $cur =~ /v|\..*\./ or $min =~ /v|\..*\./) {
+	if ($version::VERSION or defined(_load('version'))) {
+	    # use version.pm if it is installed.
+	    return ((version->new($cur) >= version->new($min)) ? $cur : undef);
+	}
+	elsif ($Sort::Versions::VERSION or defined(_load('Sort::Versions'))) {
+	    # use Sort::Versions as the sorting algorithm for a.b.c versions
+	    return ((Sort::Versions::versioncmp($cur, $min) != -1) ? $cur : undef);
+	}
+
+	warn "Cannot reliably compare non-decimal formatted versions.\n".
+	     "Please install version.pm or Sort::Versions.\n";
     }
 
     # plain comparison
@@ -680,7 +717,6 @@ sub Write {
 	return;
     }
 
-
     my %args = @_;
 
     $args{PREREQ_PM} = { %{$args{PREREQ_PM} || {} }, @Existing, @Missing }
@@ -690,14 +726,14 @@ sub Write {
 	require ExtUtils::Manifest;
 	my $manifest = ExtUtils::Manifest::maniread('MANIFEST');
 
-	$args{EXE_FILES} = [ 
-	    grep { exists $manifest->{$_} } @{$args{EXE_FILES}} 
+	$args{EXE_FILES} = [
+	    grep { exists $manifest->{$_} } @{$args{EXE_FILES}}
 	];
     }
 
     $args{test}{TESTS} ||= 't/*.t';
     $args{test}{TESTS} = join(' ', grep {
-	!exists($DisabledTests{$_}) 
+	!exists($DisabledTests{$_})
     } map { glob($_) } split(/\s+/, $args{test}{TESTS}));
 
     my $missing = join(',', @Missing);
@@ -705,17 +741,30 @@ sub Write {
 	UNIVERSAL::isa($Config, 'HASH') ? %{$Config} : @{$Config}
     ) if $Config;
 
-    my $action = (
+    $PostambleActions = (
 	$missing ? "\$(PERL) $0 --config=$config --installdeps=$missing"
 		 : "\$(NOOP)"
     );
 
     no strict 'refs';
 
-    my $old_postamble = defined(&MY::postamble) ? \&MY::postamble : sub { '' };
+    $PostambleUsed = 0;
+    local *MY::postamble = \&postamble unless defined &MY::postamble;
+    ExtUtils::MakeMaker::WriteMakefile(%args);
 
-    *{'MY::postamble'} = sub {
-	return &{$old_postamble} . << ".";
+    print << "." unless $PostambleUsed;
+*** WARNING: Makefile written with customized MY::postamble() without
+    including contents from ExtUtils::AutoInstall::postamble() --
+    auto installation features disabled.  Please contact the author.
+.
+
+    return 1;
+}
+
+sub postamble {
+    $PostambleUsed = 1;
+
+    return << ".";
 
 config :: installdeps
 
@@ -723,13 +772,9 @@ checkdeps ::
 	\$(PERL) $0 --checkdeps
 
 installdeps ::
-	$action
+	$PostambleActions
 .
-    } unless $TestOnly;
 
-    ExtUtils::MakeMaker::WriteMakefile(%args);
-
-    return 1;
 }
 
 1;
@@ -745,25 +790,10 @@ L<CPANPLUS>
 
 The test script included in the B<ExtUtils::AutoInstall> distribution
 contains code adapted from Michael Schwern's B<Test::More> under the
-I<Artistic License>.  Please refer to F<t/AutoInstall.t> for details.
+I<Perl License>.  Please consult to F<t/AutoInstall.t> for details.
 
-Thanks also to Jesse Vincent for suggesting the semantics of various
-F<make> targets, and Jos IBoumans for introducing me to his B<CPANPLUS>
-project.
-
-Eric Andreychek contributed to the C<-force> pseudo-option feature;
-Brian Ingerson suggested the non-intrusive handling of C<-core> and
-bootstrap installations, and let the user have total control.
-
-Bryan Dumm suggested the pre-/post-install handler features which
-makes up the 0.40 release.
-
-Rocco Caputo made me write compatibility code for F<cpansmoke> and
-other non-tty STDIN type installations.  Matt Cashner suggested the
-C<-skiptest> semantic and caught a subtle bug involving C<require>
-instead of C<use> of AutoInstall.  Chia-Liang Kao spotted the
-incompatibility between the use of C<$0> and CPANPLUS's C<eval()>
-munging.  Thanks!
+See the F<AUTHORS> file in this module's source distribution for the
+list of contributors.
 
 =head1 AUTHORS
 
@@ -773,7 +803,7 @@ Autrijus Tang E<lt>autrijus@autrijus.orgE<gt>
 
 Copyright 2001, 2002 by Autrijus Tang E<lt>autrijus@autrijus.orgE<gt>.
 
-This program is free software; you can redistribute it and/or 
+This program is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
 
 See L<http://www.perl.com/perl/misc/Artistic.html>
